#!/usr/bin/env python

"""Prints the PROD and DEV schema urls.

Enables export of schema urls out of this project.
"""

import sys

sys.path.append("src/fmu/datamodels/")
from _schema_urls import FmuSchemaUrls  # type: ignore

print(f"PROD_SCHEMA_URL={FmuSchemaUrls.PROD_URL}")
print(f"DEV_SCHEMA_URL={FmuSchemaUrls.DEV_URL}")



from __future__ import annotations

import gc
from pathlib import Path

import pytest

import fmu.datamodels as models
from fmu.datamodels._schema_base import FMU_SCHEMAS_PATH, SchemaBase


def test_schemabase_validates_class_vars() -> None:
    """Tests that light validation on the schema base class functions."""
    with pytest.raises(TypeError, match="Subclass A must define 'PATH'"):

        class A(SchemaBase):
            VERSION: str = "0.8.0"
            VERSION_CHANGELOG: str = "### 0.8.0"
            FILENAME: str = "fmu_results.json"

    with pytest.raises(TypeError, match="Subclass B must define 'FILENAME'"):

        class B(SchemaBase):
            VERSION: str = "0.8.0"
            VERSION_CHANGELOG: str = "### 0.8.0"
            PATH: Path = FMU_SCHEMAS_PATH / "test"

    with pytest.raises(TypeError, match="Subclass C must define 'VERSION'"):

        class C(SchemaBase):
            VERSION_CHANGELOG: str = "### 0.8.0"
            FILENAME: str = "fmu_results.json"
            PATH: Path = FMU_SCHEMAS_PATH / "test"

    with pytest.raises(TypeError, match="Subclass D must define 'VERSION_CHANGELOG'"):

        class D(SchemaBase):
            VERSION: str = "0.8.0"
            FILENAME: str = "fmu_results.json"
            PATH: Path = FMU_SCHEMAS_PATH / "test"


def test_schemabase_validates_version_string_form() -> None:
    """Tests that the VERSION given to the schema raises if not of a valid form."""
    with pytest.raises(TypeError, match="Invalid VERSION format for 'MajorMinor'"):

        class MajorMinor(SchemaBase):
            VERSION = "12.3"
            VERSION_CHANGELOG: str = "### 12.3"
            FILENAME: str = "fmu_results.json"
            PATH: Path = FMU_SCHEMAS_PATH / "test"

    with pytest.raises(TypeError, match="Invalid VERSION format for 'Alphanumeric'"):

        class Alphanumeric(SchemaBase):
            VERSION = "1.3.a"
            VERSION_CHANGELOG: str = "### 1.3.a"
            FILENAME: str = "fmu_results.json"
            PATH: Path = FMU_SCHEMAS_PATH / "test"

    with pytest.raises(TypeError, match="Invalid VERSION format for 'LeadingZero'"):

        class LeadingZero(SchemaBase):
            VERSION = "01.3.0"
            VERSION_CHANGELOG: str = "### 01.3.0"
            FILENAME: str = "fmu_results.json"
            PATH: Path = FMU_SCHEMAS_PATH / "test"


def test_schemabase_requires_path_starting_with_fmuschemas_path() -> None:
    """Tests that SchemaBase catches if a subclass's PATH does not fall into the main
    schemas directory."""
    with pytest.raises(
        ValueError, match=f"PATH must start with `FMU_SCHEMAS_PATH`: {FMU_SCHEMAS_PATH}"
    ):

        class A(SchemaBase):
            VERSION: str = "0.8.0"
            VERSION_CHANGELOG: str = "### 0.8.0"
            FILENAME: str = "fmu_results.json"
            PATH: Path = Path("test")


def test_schemabase_validates_a_version_has_a_changelog_entry() -> None:
    """Tests that a version change has a corresponding changelog entry."""
    with pytest.raises(
        ValueError, match="No changelog entry exists for 'A' version 0.9.0"
    ):

        class A(SchemaBase):
            VERSION: str = "0.9.0"
            VERSION_CHANGELOG: str = ""
            FILENAME: str = "fmu_results.json"
            PATH: Path = FMU_SCHEMAS_PATH / "test"

    # Should not raise ValueError
    class B(SchemaBase):
        VERSION: str = "0.9.0"
        VERSION_CHANGELOG: str = "### 0.9.0"
        FILENAME: str = "fmu_results.json"
        PATH: Path = FMU_SCHEMAS_PATH / "test"


def test_schemabase_all_subclasses_in_exposed_schemas_list() -> None:
    """Tests that all schemas are in the list of schemas to generate and expose."""
    # Make sure test classes are removed as subclasses of SchemaBase
    gc.collect()

    schemas = SchemaBase.__subclasses__()
    exposed_schemas = models.schemas
    assert len(schemas) == len(exposed_schemas)
    assert set(schemas) == set(exposed_schemas)



"""Top-level package for fmu-datamodels."""

from fmu.datamodels._schema_base import SchemaBase

from .fmu_results import FmuResults, FmuResultsSchema
from .standard_results import (
    ErtDistribution,
    ErtParameterMetadata,
    ErtParametersResult,
    ErtParametersSchema,
    FieldOutlineResult,
    FieldOutlineSchema,
    FluidContactOutlineResult,
    FluidContactOutlineSchema,
    InplaceVolumesResult,
    InplaceVolumesSchema,
    SimulatorFipregionsMappingResult,
    SimulatorFipregionsMappingSchema,
    StructureDepthFaultLinesResult,
    StructureDepthFaultLinesSchema,
)

try:
    from .version import version

    __version__ = version
except ImportError:
    __version__ = "0.0.0"

__all__ = [
    "ErtDistribution",
    "ErtParameterMetadata",
    "ErtParametersResult",
    "ErtParametersSchema",
    "FmuResults",
    "FmuResultsSchema",
    "FieldOutlineResult",
    "FieldOutlineSchema",
    "FluidContactOutlineResult",
    "FluidContactOutlineSchema",
    "InplaceVolumesResult",
    "InplaceVolumesSchema",
    "SimulatorFipregionsMappingResult",
    "SimulatorFipregionsMappingSchema",
    "StructureDepthFaultLinesResult",
    "StructureDepthFaultLinesSchema",
]

schemas: list[type[SchemaBase]] = [
    FmuResultsSchema,
    ErtParametersSchema,
    FieldOutlineSchema,
    FluidContactOutlineSchema,
    InplaceVolumesSchema,
    SimulatorFipregionsMappingSchema,
    StructureDepthFaultLinesSchema,
]



import pytest
from pydantic import ValidationError

from fmu.datamodels.fmu_results.fmu_results import FmuResults, FmuResultsSchema


def test_version_string_type(fluid_contact_metadata: dict) -> None:
    """Tests fmu.datamodels.types.VerionStr."""

    # assert validation as-is
    FmuResults.model_validate(fluid_contact_metadata)

    # assert validation fails when version pattern is invalid
    fluid_contact_metadata["version"] = "1.2.a"
    with pytest.raises(ValidationError, match="String should match pattern"):
        FmuResults.model_validate(fluid_contact_metadata)

    fluid_contact_metadata["version"] = "1.2"
    with pytest.raises(ValidationError, match="String should match pattern"):
        FmuResults.model_validate(fluid_contact_metadata)

    # assert version default if version not set
    del fluid_contact_metadata["version"]
    assert "version" not in fluid_contact_metadata
    model = FmuResults.model_validate(fluid_contact_metadata)
    assert model.root.version == FmuResultsSchema.VERSION



from __future__ import annotations

from typing import Annotated, Literal

from pydantic import (
    AnyHttpUrl,
    BaseModel,
    Field,
    RootModel,
)

from fmu.datamodels.standard_results import (
    ErtParametersSchema,
    FieldOutlineSchema,
    FluidContactOutlineSchema,
    InplaceVolumesSchema,
    SimulatorFipregionsMappingSchema,
    StandardResultName,
    StructureDepthFaultLinesSchema,
)
from fmu.datamodels.types import VersionStr


class FileSchema(BaseModel):
    """The schema identifying the format of a standard result."""

    version: VersionStr
    """The version of the standard result schema."""

    url: AnyHttpUrl
    """The url to the standard result schema."""


class StandardResult(BaseModel):
    """
    The ``standard_result`` field contains information about which standard result this
    data object represents.
    """

    name: StandardResultName
    """The identifying standard result name for this data object."""

    file_schema: FileSchema | None = Field(default=None)
    """The schema identifying the format of the standard result."""


class ErtParametersStandardResult(StandardResult):
    """
    The ``standard_result`` field contains information about which standard results this
    data object represents.

    This class contains metadata for the 'parameters' standard result.
    """

    name: Literal[StandardResultName.parameters]
    """The identifying name for the 'parameters' standard result."""

    file_schema: FileSchema = FileSchema(
        version=ErtParametersSchema.VERSION,
        url=AnyHttpUrl(ErtParametersSchema.url()),
    )
    """The schema identifying the format of the 'parameters' standard result."""


class InplaceVolumesStandardResult(StandardResult):
    """
    The ``standard_result`` field contains information about which standard results this
    data object represents.

    This class contains metadata for the 'inplace_volumes' standard result.
    """

    name: Literal[StandardResultName.inplace_volumes]
    """The identifying name for the 'inplace_volumes' standard result."""

    file_schema: FileSchema = FileSchema(
        version=InplaceVolumesSchema.VERSION,
        url=AnyHttpUrl(InplaceVolumesSchema.url()),
    )
    """The schema identifying the format of the 'inplace_volumes' standard result."""


class SimulatorFipregionsMappingStandardResult(StandardResult):
    """
    The ``standard_result`` field contains information about which standard results this
    data object represents.

    This class contains metadata for the 'simulator_fipregions_mapping'
    standard result.
    """

    name: Literal[StandardResultName.simulator_fipregions_mapping]
    """The identifying name for the 'simulator_fipregions_mapping' standard result."""

    file_schema: FileSchema = FileSchema(
        version=SimulatorFipregionsMappingSchema.VERSION,
        url=AnyHttpUrl(SimulatorFipregionsMappingSchema.url()),
    )
    """
    The schema identifying the format of the 'simulator_fipregions_mapping'
    standard result.
    """


class StructureDepthSurfaceStandardResult(StandardResult):
    """
    The ``standard_result`` field contains information about which standard results this
    data object represent.
    This class contains metadata for the 'structure_depth_surface' standard result.
    """

    name: Literal[StandardResultName.structure_depth_surface]
    """The identifying name for the 'structure_depth_surface' standard result."""


class StructureDepthFaultSurfaceStandardResult(StandardResult):
    """
    The ``standard_result`` field contains information about which standard results this
    data object represent.
    This class contains metadata for the 'structure_depth_fault_surface'
    standard result.
    """

    name: Literal[StandardResultName.structure_depth_fault_surface]
    """The identifying name for the 'structure_depth_fault_surface' standard result."""


class StructureTimeSurfaceStandardResult(StandardResult):
    """
    The ``standard_result`` field contains information about which standard results this
    data object represent.
    This class contains metadata for the 'structure_time_surface' standard result.
    """

    name: Literal[StandardResultName.structure_time_surface]
    """The identifying name for the 'structure_time_surface' standard result."""


class GridExtractedDepthSurfaceStandardResult(StandardResult):
    """
    The ``standard_result`` field contains information about which standard results this
    data object represent.
    This class contains metadata for the "grid_extracted_depth_surface' standard result.
    """

    name: Literal[StandardResultName.grid_extracted_depth_surface]
    """The identifying name for the 'grid_extracted_depth_surface' standard result."""


class StructureDepthIsochoreStandardResult(StandardResult):
    """
    The ``standard_result`` field contains information about which standard results this
    data object represent.
    This class contains metadata for the 'structure_depth_isochore' standard result.
    """

    name: Literal[StandardResultName.structure_depth_isochore]
    """The identifying name for the 'structure_depth_isochore' standard result."""


class StructureDepthFaultLinesStandardResult(StandardResult):
    """
    The ``standard_result`` field contains information about which standard results this
    data object represent.
    This class contains metadata for the 'structure_depth_fault_lines' standard result.
    """

    name: Literal[StandardResultName.structure_depth_fault_lines]
    """The identifying name for the 'structure_depth_fault_lines' standard result."""

    file_schema: FileSchema = FileSchema(
        version=StructureDepthFaultLinesSchema.VERSION,
        url=AnyHttpUrl(StructureDepthFaultLinesSchema.url()),
    )
    """
    The schema identifying the format of the 'structure_depth_fault_lines'
    standard result.
    """


class FieldOutlineStandardResult(StandardResult):
    """
    The ``standard_result`` field contains information about which standard results this
    data object represent.
    This class contains metadata for the 'field_outline' standard result.
    """

    name: Literal[StandardResultName.field_outline]
    """The identifying name for the 'field_outline' standard result."""

    file_schema: FileSchema = FileSchema(
        version=FieldOutlineSchema.VERSION,
        url=AnyHttpUrl(FieldOutlineSchema.url()),
    )
    """
    The schema identifying the format of the 'field_outline' standard result.
    """


class FluidContactSurfaceStandardResult(StandardResult):
    """
    The ``standard_result`` field contains information about which standard results this
    data object represent.
    This class contains metadata for the 'fluid_contact_surface' standard result.
    """

    name: Literal[StandardResultName.fluid_contact_surface]
    """The identifying name for the 'fluid_contact_surface' standard result."""


class FluidContactOutlineStandardResult(StandardResult):
    """
    The ``standard_result`` field contains information about which standard results this
    data object represent.
    This class contains metadata for the 'fluid_contact_outline' standard result.
    """

    name: Literal[StandardResultName.fluid_contact_outline]
    """The identifying name for the 'fluid_contact_outline' standard result."""

    file_schema: FileSchema = FileSchema(
        version=FluidContactOutlineSchema.VERSION,
        url=AnyHttpUrl(FluidContactOutlineSchema.url()),
    )
    """
    The schema identifying the format of the 'fluid_contact_outline' standard result.
    """


class LiftCurvesStandardResult(StandardResult):
    """This class contains metadata for the 'lift_curves' standard result."""

    name: Literal[StandardResultName.lift_curves]
    """The identifying name for the 'lift_curves' standard result."""


class ProductionNetworkStandardResult(StandardResult):
    """This class contains metadata for the 'production_network' standard result."""

    name: Literal[StandardResultName.production_network]
    """The identifying name for the 'production_network' standard result."""


class PvtStandardResult(StandardResult):
    """This class contains metadata for the 'pvt' standard result."""

    name: Literal[StandardResultName.pvt]
    """The identifying name for the 'pvt' standard result."""


class RelpermStandardResult(StandardResult):
    """This class contains metadata for the 'relperm' standard result."""

    name: Literal[StandardResultName.relperm]
    """The identifying name for the 'relperm' standard result."""


class RftStandardResult(StandardResult):
    """This class contains metadata for the 'rft' standard result."""

    name: Literal[StandardResultName.rft]
    """The identifying name for the 'rft' standard result."""


class SimulationTimeseriesStandardResult(StandardResult):
    """This class contains metadata for the 'simulationtimeseries' standard result."""

    name: Literal[StandardResultName.simulationtimeseries]
    """The identifying name for the 'simulationtimeseries' standard result."""


class TransmissibilitiesStandardResult(StandardResult):
    """This class contains metadata for the 'transmissibilities' standard result."""

    name: Literal[StandardResultName.transmissibilities]
    """The identifying name for the 'transmissibilities' standard result."""


class WellCompletionsStandardResult(StandardResult):
    """This class contains metadata for the 'well_completions' standard result."""

    name: Literal[StandardResultName.well_completions]
    """The identifying name for the 'well_completions' standard result."""


class AnyStandardResult(RootModel):
    """
    The ``standard result`` field contains information about which standard result this
    data object represents. Data that is tagged as such is a standard result from FMU
    that conforms to a specified standard.

    This class, ``AnyStandardResult``, acts as a container for different standard
    results, with the exact standard result being identified by the
    ``standard_result.name`` field.
    """

    root: Annotated[
        ErtParametersStandardResult
        | FieldOutlineStandardResult
        | InplaceVolumesStandardResult
        | SimulatorFipregionsMappingStandardResult
        | StructureDepthSurfaceStandardResult
        | StructureDepthFaultSurfaceStandardResult
        | StructureTimeSurfaceStandardResult
        | GridExtractedDepthSurfaceStandardResult
        | StructureDepthIsochoreStandardResult
        | StructureDepthFaultLinesStandardResult
        | FluidContactSurfaceStandardResult
        | FluidContactOutlineStandardResult
        | LiftCurvesStandardResult
        | ProductionNetworkStandardResult
        | PvtStandardResult
        | RelpermStandardResult
        | RftStandardResult
        | SimulationTimeseriesStandardResult
        | TransmissibilitiesStandardResult
        | WellCompletionsStandardResult,
        Field(discriminator="name"),
    ]



from __future__ import annotations

from uuid import UUID

from pydantic import (
    BaseModel,
    Field,
)


class Masterdata(BaseModel):
    """The ``masterdata`` block contains information related to masterdata.
    Currently, SMDA holds the masterdata.
    """

    smda: Smda
    """Block containing SMDA-related attributes. See :class:`Smda`."""


class Smda(BaseModel):
    """The ``masterdata.smda`` block contains SMDA-related attributes."""

    coordinate_system: CoordinateSystem
    """Reference to coordinate system known to SMDA.
    See :class:`CoordinateSystem`."""

    country: list[CountryItem]
    """A list referring to countries known to SMDA. First item is primary.
    See :class:`CountryItem`."""

    discovery: list[DiscoveryItem]
    """A list referring to discoveries known to SMDA. First item is primary.
    See :class:`DiscoveryItem`."""

    field: list[FieldItem]
    """A list referring to fields known to SMDA. First item is primary.
    See :class:`FieldItem`."""

    stratigraphic_column: StratigraphicColumn
    """Reference to stratigraphic column known to SMDA.
    See :class:`StratigraphicColumn`."""


class CountryItem(BaseModel):
    """A single country in the ``smda.masterdata.country`` list of countries
    known to SMDA."""

    identifier: str = Field(examples=["Norway"])
    """Identifier known to SMDA."""

    uuid: UUID = Field(examples=["15ce3b84-766f-4c93-9050-b154861f9100"])
    """Identifier known to SMDA."""


class DiscoveryItem(BaseModel):
    """A single discovery in the ``masterdata.smda.discovery`` list of discoveries
    known to SMDA."""

    short_identifier: str = Field(examples=["SomeDiscovery"])
    """Identifier known to SMDA."""

    uuid: UUID = Field(examples=["15ce3b84-766f-4c93-9050-b154861f9100"])
    """Identifier known to SMDA."""


class FieldItem(BaseModel):
    """A single field in the ``masterdata.smda.field`` list of fields
    known to SMDA."""

    identifier: str = Field(examples=["OseFax"])
    """Identifier known to SMDA."""

    uuid: UUID = Field(examples=["15ce3b84-766f-4c93-9050-b154861f9100"])
    """Identifier known to SMDA."""


class CoordinateSystem(BaseModel):
    """The ``masterdata.smda.coordinate_system`` block contains the coordinate
    system known to SMDA."""

    identifier: str = Field(examples=["ST_WGS84_UTM37N_P32637"])
    """Identifier known to SMDA."""

    uuid: UUID = Field(examples=["15ce3b84-766f-4c93-9050-b154861f9100"])
    """Identifier known to SMDA."""


class StratigraphicColumn(BaseModel):
    """The ``masterdata.smda.stratigraphic_column`` block contains the
    stratigraphic column known to SMDA."""

    identifier: str = Field(examples=["DROGON_2020"])
    """Identifier known to SMDA."""

    uuid: UUID = Field(examples=["15ce3b84-766f-4c93-9050-b154861f9100"])
    """Identifier known to SMDA."""



from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING

from pydantic import BaseModel, Field, RootModel

from fmu.datamodels._schema_base import FMU_SCHEMAS_PATH, SchemaBase
from fmu.datamodels.types import VersionStr

if TYPE_CHECKING:
    from typing import Any


class SimulatorFipregionsMappingResultRow(BaseModel):
    """Represents the columns of a row in a simulator fipregions mapping export.

    These fields are the current agreed upon standard result. Changes to the fields or
    their validation should cause the version defined in the standard result schema to
    increase the version number in a way that corresponds to the schema versioning
    specification (i.e. they are a patch, minor, or major change)."""

    FIPNUM: int = Field(ge=0)
    """Index column. The id of the fipregion this row represents. Required."""

    ZONE: str
    """Index column. The zone that the fipregion corresponds to. Required."""

    REGION: str
    """Index column. The region that the fipregion corresponds to. Required."""


class SimulatorFipregionsMappingResult(RootModel):
    """Represents the resultant simulator fipregions mapping parquet file, which is
    naturally a list of rows.

    Consumers who retrieve this parquet file must read it into a json-dictionary
    equivalent format to validate it against the schema."""

    root: list[SimulatorFipregionsMappingResultRow]


class SimulatorFipregionsMappingSchema(SchemaBase):
    """This class represents the schema that is used to validate the mapping
    table being exported. This means that the version, schema filename, and
    schema location corresponds directly with the values and their validation
    constraints, documented above."""

    VERSION: VersionStr = "0.1.0"

    VERSION_CHANGELOG: str = """
    #### 0.1.0

    This is the initial schema version.
    """

    FILENAME: str = "simulator_fipregions_mapping.json"
    """The filename this schema is written to."""

    PATH: Path = FMU_SCHEMAS_PATH / "file_formats" / VERSION / FILENAME
    """The local and URL path of this schema."""

    @classmethod
    def dump(cls) -> dict[str, Any]:
        return SimulatorFipregionsMappingResult.model_json_schema(
            schema_generator=cls.default_generator()
        )



from __future__ import annotations

from enum import StrEnum


class Classification(StrEnum):
    """The security classification for a given data object."""

    asset = "asset"
    # Deprecated. This value should be undocumented and eventually removed.

    internal = "internal"
    """Grants access to all users with ``READ`` access to the asset.

    The ``READ`` role is an access role defined by the asset's Unix and Sumo groups.
    This is the default for most data.
    """

    restricted = "restricted"
    """Grants access to all users with ``WRITE`` access to the asset.

    The ``WRITE`` role is an access role defined by the asset's Unix and Sumo groups.
    This is the default for some sensitive data, like volumes, but in general must be
    explicitly set when restricted access is desired.
    """


class TrackLogEventType(StrEnum):
    """The type of event being logged"""

    created = "created"
    """The initial data was created."""

    updated = "updated"
    """Data was updated."""

    merged = "merged"
    """Data was merged."""



from __future__ import annotations

from enum import IntEnum, StrEnum


class AxisOrientation(IntEnum):
    """The axis orientation for a given data object."""

    normal = 1
    """A coordinate system with Z-axis positive downwards."""

    flipped = -1
    """A coordinate system with Z-axis positive upwards."""


class Content(StrEnum):
    """The content type of a given data object."""

    depth = "depth"
    """A data object representing depth values.

    Typically provided as an ``xtgeo.RegularSurface`` or ``xtgeo.Grid`` for export.
    """

    facies_thickness = "facies_thickness"
    """Thickness map representing facies thickness, derived from a 3D grid.

    Typically provided as an ``xtgeo.RegularSurface`` for export.
    """

    fault_lines = "fault_lines"
    """Intersections between fault planes and horizons.

    Typically provided as an ``xtgeo.Polygons`` for export.
    """

    fault_surface = "fault_surface"
    """A surface representing a fault plane.

    Typically provided either as an RMS FaultRoom GeoJSON surface or an fmu-dataio
    ``TSurfData`` for export.
    """

    fault_properties = "fault_properties"
    """Properties, such as permeability and porosity, on a fault.

    Typically provided as a GeoJSON file derived from RMS FaultRoom for export.
    """

    field_outline = "field_outline"
    """Polygons representing the outline of a field, initial (static) conditions.

    Typically provided as an ``xtgeo.Polygons`` for export.
    """

    field_region = "field_region"
    """Delineated or named region within a field.

    Typically provided as an ``xtgeo.Polygons`` for export.
    """

    fluid_contact = "fluid_contact"
    """Depth surface representing a fluid contact used per realization.

    Typically provided as an ``xtgeo.RegularSurface`` for export.
    """

    khproduct = "khproduct"
    """The product of permeability (k) and reservoir thickness (h).

    Typically provided as an ``xtgeo.RegularSurface`` for export.
    """

    lift_curves = "lift_curves"
    """Table representing the relationship between production rates and pressures.

    Typically provided as a Pandas ``Dataframe`` for export.
    """
    mapping = "mapping"
    """Tabular cross-references used to translate between different naming conventions
    or identifiers.

    Acts as a bridge to align data across different domains, such as:
    * Official stratigraphy to model zonation.
    * Static reservoir regions/zones to simulator-specific identifiers (e.g., FIPGRP).
    * Unique Well Identifiers (UWI) to simulation well names.

    Typically provided as a Pandas ``Dataframe`` for export.
    """

    named_area = "named_area"
    """A named area within a field that is _not_ a region.

    Typically provided as an ``xtgeo.Polygons`` for export.
    """

    parameters = "parameters"
    """The ERT parameters generated for the realization.

    Typically provided as a Pandas ``Dataframe`` for export.

    .. tip::

       You should not export this manually. This is done automatically by `SUMO_UPLOADER
       <https://fmu-sumo-uploader.readthedocs.io/en/latest/>`_ when uploading files.

    """

    production_network = "production_network"
    """Tabular data representing the production group structure.

    Typically provided as a Pandas ``Dataframe``.

    .. tip::

       You should not export this manually. Use `SIM2SUMO
       <https://fmu-sumo-sim2sumo.readthedocs.io/en/latest/>`_.
    """

    pinchout = "pinchout"
    """Polygons designating a pinchout.

    Typically provided as an ``xtgeo.Polygons`` for export.
    """

    property = "property"
    """A property, like permeability or porosity, belonging to a 3D grid.

    Typically provided as an ``xtgeo.GridProperty``.

    .. tip::

       This content type requires additional input in the ``content_metadata`` field.

       Grid property data handling is still immature. More comprehensive data
       categorization will come in the future.

    """

    pvt = "pvt"
    """Tabular pressure-volume-temperature data.

    Typically provided as a Pandas ``Dataframe`` for export.

    .. tip::

       You should not export this manually. Use `SIM2SUMO
       <https://fmu-sumo-sim2sumo.readthedocs.io/en/latest/>`_.

    """

    regions = "regions"
    """Distinct areas within the field that have different characteristics.

    Examples may be volume regions or contact regions.

    Typically provided as an ``xtgeo.Polygons`` or ``xtgeo.GridProperty``.
    """

    relperm = "relperm"
    """Tabular relative permeability data.

    Typically provided as a Pandas ``Dataframe`` for export.

    .. tip::

       You should not export this manually. Use `SIM2SUMO
       <https://fmu-sumo-sim2sumo.readthedocs.io/en/latest/>`_.

    """

    rft = "rft"
    """Tabular reservoir formation tests data.

    .. tip::

       You should not export this manually. Use `SIM2SUMO
       <https://fmu-sumo-sim2sumo.readthedocs.io/en/latest/>`_.

    """

    seismic = "seismic"
    """Data that is seismic in nature, including seismic cubes and surface data derived
    from seismic cubes.

    Typically provided as an ``xtgeo.Cube``, ``xtgeo.RegularSurface``, or other.

    .. tip::

       This content type requires additional input in the ``content_metadata`` field.

       Seismic data handling is still immature. More comprehensive data categorization
       will come in the future.

    """

    simulationtimeseries = "simulationtimeseries"
    """Time-series data generated by a reservoir simulator like OPM Flow or Eclipse.

    For example, a summary file parsed into a Pandas ``Dataframe`` by
    `res2df <https://equinor.github.io/res2df>`_.

    .. tip::

       You should not export this manually. Use `SIM2SUMO
       <https://fmu-sumo-sim2sumo.readthedocs.io/en/latest/>`_.

    """

    subcrop = "subcrop"
    """Surface or polygon representing a subcrop area.

    Typically provided as an ``xtgeo.RegularSurface`` or ``xtgeo.Polygons`` for export.
    """

    thickness = "thickness"
    """A thickness map.

    Typically provided as an ``xtgeo.RegularSurface`` for export.
    """

    time = "time"
    """A seismic time surface or seismic cube in time domain.

    Typically provided as an ``xtgeo.RegularSurface`` or ``xtgeo.Cube``.
    """

    timeseries = "timeseries"
    # Deprecated. Use "simlationtimeseries" ?

    transmissibilities = "transmissibilities"
    """Tabular data containing transmissibilities (neighbour and
    non-neigbor-connections).

    Typically provided as a Pandas ``Dataframe``.

    .. tip::

       You should not export this manually. Use `SIM2SUMO
       <https://fmu-sumo-sim2sumo.readthedocs.io/en/latest/>`_.

    """

    velocity = "velocity"
    """A seismic velocity map represented as a regular surface or a cube.

    Typically provided as an ``xtgeo.RegularSurface`` or ``xtgeo.Cube`` for export.
    """

    volumes = "volumes"
    """Tabulated inplace volumes per grid, initial (static) conditions.

    Typically provided as a Pandas ``Dataframe``.
    """

    well_completions = "well_completions"
    """Tabular data representing well completions.

    Typically provided as a Pandas ``Dataframe``.

    .. tip::

       You should not export this manually. Use `SIM2SUMO
       <https://fmu-sumo-sim2sumo.readthedocs.io/en/latest/>`_.

    """

    wellpicks = "wellpicks"
    """Tabular data representing wellpicks.

    Typically provided as a Pandas ``Dataframe``.
    """

    @classmethod
    def _missing_(cls: type[Content], value: object) -> None:
        raise ValueError(
            f"Invalid 'content' {value=}. Valid entries are {[m.value for m in cls]}"
        )


class ErtSimulationMode(StrEnum):
    """The simulation mode ert was run in. These definitions come from
    `ert.mode_definitions`."""

    ensemble_experiment = "ensemble_experiment"
    ensemble_information_filter = "ensemble_information_filter"
    ensemble_smoother = "ensemble_smoother"
    es_mda = "es_mda"
    evaluate_ensemble = "evaluate_ensemble"
    manual_enif_update = "manual_enif_update"
    manual_update = "manual_update"
    test_run = "test_run"
    workflow = "workflow"


class MetadataClass(StrEnum):
    """Base class for objects by FMU convention or standards."""


class ObjectMetadataClass(MetadataClass):
    """The class of a data object (typically originating from an RMS model)."""

    surface = "surface"
    table = "table"
    cpgrid = "cpgrid"
    cpgrid_property = "cpgrid_property"
    polygons = "polygons"
    cube = "cube"
    well = "well"
    points = "points"
    dictionary = "dictionary"


class FMUResultsMetadataClass(MetadataClass):
    """The class of an FMU results object."""

    case = "case"
    realization = "realization"
    iteration = "iteration"
    ensemble = "ensemble"


class Layout(StrEnum):
    """The layout of a given data object."""

    regular = "regular"
    unset = "unset"
    cornerpoint = "cornerpoint"
    table = "table"
    dictionary = "dictionary"
    triangulated = "triangulated"


class FMUContext(StrEnum):
    """The context in which FMU was being run when data were generated."""

    case = "case"
    iteration = "iteration"
    ensemble = "ensemble"
    realization = "realization"


class VerticalDomain(StrEnum):
    depth = "depth"
    """In the domain of depth."""

    time = "time"
    """In the domain of time."""


class DomainReference(StrEnum):
    msl = "msl"
    """In reference to Mean Sea Level."""

    sb = "sb"
    """In reference to Sea Bottom."""

    rkb = "rkb"
    """In reference to Rotary Kelly Bushing (RKB)."""


class FluidContactType(StrEnum):
    """The type of fluid contact."""

    fgl = "fgl"
    """Free gas level."""

    fwl = "fwl"
    """Free water level."""

    goc = "goc"
    """Gas-oil contact."""

    gwc = "gwc"
    """Gas-water contact."""

    owc = "owc"
    """Oil-water contact."""


class FileFormat(StrEnum):
    """The format of a given data object."""

    parquet = "parquet"
    json = "json"
    csv = "csv"
    csv_xtgeo = "csv|xtgeo"
    irap_ascii = "irap_ascii"
    irap_binary = "irap_binary"
    roff = "roff"
    segy = "segy"
    openvds = "openvds"
    tsurf = "tsurf"
